==> newsstar-1.2.2.tar.gz: newsstar-1.2.2/src/groupcount.c <==
/*
    newsstar - advanced news fetcher
    Copyright (C) 2001-2002 Tony Houghton <h@realh.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* Keep track of how many articles are to be downloaded for each group across
 * all servers */

/* $Id: groupcount.c,v 1.2 2004/08/23 00:47:49 realh Exp $ */

#include "groupcount.h"
#include "group.h"
#include "server.h"
#include "stdx.h"

struct GroupCountData {
    char *name;
    int n_servers;
    int n_articles;
};

static GroupCountData **groupcount_vec = NULL;
static size_t groupcount_n_items = 0;
static size_t groupcount_vec_size = 0;

/* Adds a new group to the pool or increases its server count if it already
 * exists */
GroupCountData *groupcount_add_group(const char *name)
{
    size_t n;
    GroupCountData *new_gc;

    for (n = 0; n < groupcount_n_items; ++n)
    {
	if (!strcmp(name, groupcount_vec[n]->name))
	{
	    ++groupcount_vec[n]->n_servers;
	    return groupcount_vec[n];
	}
    }
    if (groupcount_n_items + 1 > groupcount_vec_size)
    {
	groupcount_vec_size = groupcount_vec_size ? groupcount_vec_size * 2 : 8;
	groupcount_vec = XREALLOC(groupcount_vec,
		sizeof(GroupCountData *) * groupcount_vec_size);

    }
    new_gc = XMALLOC(sizeof(GroupCountData));
    groupcount_vec[groupcount_n_items++] = new_gc;
    new_gc->name = XSTRDUP(name);
    new_gc->n_servers = 1;
    new_gc->n_articles = 0;
    return new_gc;
}

void groupcount_inc_articles(GroupCountData *gc)
{
    ++gc->n_articles;
}

int groupcount_server_done_get_total(GroupCountData *gc)
{
    return --gc->n_servers ? -1 : gc->n_articles;
}

==> newsstar-1.5.3.tar.gz: newsstar-1.5.3/src/groupcount.c <==
/*
    newsstar - advanced news fetcher
    Copyright (C) 2001-2002 Tony Houghton <h@realh.co.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* Keep track of how many articles are to be downloaded for each group across
 * all servers */


#include "groupcount.h"
#include "group.h"
#include "server.h"
#include "stdx.h"

struct GroupCountData {
    char *name;
    int n_servers;
    int n_articles;
};

static GroupCountData **groupcount_vec = NULL;
static size_t groupcount_n_items = 0;
static size_t groupcount_vec_size = 0;

/* Adds a new group to the pool or increases its server count if it already
 * exists */
GroupCountData *groupcount_add_group(const char *name)
{
    size_t n;
    GroupCountData *new_gc;

    for (n = 0; n < groupcount_n_items; ++n)
    {
	if (!strcmp(name, groupcount_vec[n]->name))
	{
	    ++groupcount_vec[n]->n_servers;
	    return groupcount_vec[n];
	}
    }
    if (groupcount_n_items + 1 > groupcount_vec_size)
    {
	groupcount_vec_size = groupcount_vec_size ? groupcount_vec_size * 2 : 8;
	groupcount_vec = XREALLOC(groupcount_vec,
		sizeof(GroupCountData *) * groupcount_vec_size);

    }
    new_gc = XMALLOC(sizeof(GroupCountData));
    groupcount_vec[groupcount_n_items++] = new_gc;
    new_gc->name = XSTRDUP(name);
    new_gc->n_servers = 1;
    new_gc->n_articles = 0;
    return new_gc;
}

void groupcount_inc_articles(GroupCountData *gc)
{
    ++gc->n_articles;
}

void groupcount_server_done(GroupCountData *gc)
{
    int arts = --gc->n_servers ? -1 : gc->n_articles;

    if (arts != -1)
    {
	deblog_brief(_("%4ld new %s in %s\n"), arts,
		arts == 1 ?  _("article ") : _("articles"), gc->name);
    }
}
